﻿#region Copyright
//
// (C) Copyright 2011 by Autodesk, Inc.
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted,
// provided that the above copyright notice appears in all copies and
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE. AUTODESK, INC.
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//
#endregion // Copyright

#region Namespaces
using System;
using System.Diagnostics;
using System.Collections.Generic;
using Autodesk.Revit.ApplicationServices;
using Autodesk.Revit.Attributes;
using Autodesk.Revit.DB;
using Autodesk.Revit.DB.ExtensibleStorage;
using Autodesk.Revit.UI;
using Autodesk.Revit.UI.Selection;
#endregion // Namespaces

namespace ExtensibleStorage
{
  [Transaction( TransactionMode.Manual )]
  public class Command : IExternalCommand
  {
    static string PluralSuffix( int n )
    {
      return 1 == n ? "" : "s";
    }

    /// <summary>
    /// データを壁に追加してそれを読み出します 
    /// 
    ///  
    /// </summary>
    void StoreDataInWall(
      Wall wall,
      XYZ dataToStore )
    {
      Transaction createSchemaAndStoreData
        = new Transaction( wall.Document, "tCreateAndStore" );       

      createSchemaAndStoreData.Start();

      // スキーマを作成

      SchemaBuilder schemaBuilder = new SchemaBuilder(
        new Guid( "720080CB-DA99-40DC-9415-E53F280AA1F0" ) 
        );

　　　// スキーマ名を設定

      schemaBuilder.SetSchemaName("MySchema");
      
      // 読み取りは全てのユーザに許可します

      schemaBuilder.SetReadAccessLevel(
        AccessLevel.Public 
        );

      // 書き込みはベンダのみに許可します

      schemaBuilder.SetWriteAccessLevel(
        AccessLevel.Vendor 
        );

      // 書き込み許可するベンダのIDを設定します

      schemaBuilder.SetVendorId( "ADNP" );

      // 座標を保存するためにフィールドを作成

      FieldBuilder fieldBuilder = schemaBuilder
        .AddSimpleField( "WireLocation",
        typeof( XYZ ) );

    　// 単位タイプを設定

      fieldBuilder.SetUnitType( UnitType.UT_Length );

      // フィールドの意味

      fieldBuilder.SetDocumentation( "ワイヤの位置です" );

 
      // スキーマを作成を完了

      Schema schema = schemaBuilder.Finish(); 

      // Entityオブジェトを作成

      Entity entity = new Entity( schema );

      // スキーマからフィールドを取得

      Field fieldSpliceLocation = schema.GetField(
        "WireLocation" 
        );

      // Entityオブジェクトのそのフィールドの値を設定

      entity.Set<XYZ>( fieldSpliceLocation, dataToStore,
        DisplayUnitType.DUT_METERS );

      // Entityオブジェクトを壁に追加します

      wall.SetEntity( entity );

　　　// トランザクションをコミット

      createSchemaAndStoreData.Commit();

      // 壁から今追加したデータを読み出します

　　　// スキーマを指定してEntityオブジェクトを取得

      Entity retrievedEntity = wall.GetEntity( schema );

    　// フィールドを指定して値を取得

      XYZ retrievedData = retrievedEntity.Get<XYZ>(
        schema.GetField( "WireLocation" ),
        DisplayUnitType.DUT_METERS 
        );
    }

    /// <summary>
    /// 全てのスキーマをリストします
    /// </summary>
    void ListSchemas()
    {
      IList<Schema> schemas = Schema.ListSchemas();

      int n = schemas.Count;

      Debug.Print( 
        string.Format( "{0} スキーマ{1} 定義済み:", 
        n, PluralSuffix( n ) ) 
        );

      foreach( Schema s in schemas )
      {
        IList<Field> fields = s.ListFields();

        n = fields.Count;

        Debug.Print( 
          string.Format( "スキーマ '{0}' は {1} フィールドを持ちます:", 
            s.SchemaName, n, PluralSuffix( n ) ) 
            );

        foreach( Field f in fields )
        {
          Debug.Print(
            string.Format( 
              "フィールド '{0}' は値タイプ {1} と単位タイプ {2} です",
              f.FieldName, f.ValueType, f.UnitType ) 
              );
        }
      }
    }

    class WallFilter : ISelectionFilter
    {
      public bool AllowElement( Element e )
      {
        return e is Wall;
      }

      public bool AllowReference( Reference r, XYZ p )
      {
        return true;
      }
    }

    public Result Execute(
      ExternalCommandData commandData,
      ref string message,
      ElementSet elements 
      )
    {
      UIApplication uiapp = commandData.Application;
      UIDocument uidoc = uiapp.ActiveUIDocument;
      Document doc = uidoc.Document;

      try
      {
        // 画面上で壁を選択して選択位置のXYZ座標を取得します

        Reference r = uidoc.Selection.PickObject( 
          ObjectType.Face, 
          new WallFilter(), 
          "壁のいづれかの面の点を選択してください" 
          );

        Wall wall = doc.get_Element( r.ElementId ) as Wall;
        XYZ dataToStore = r.GlobalPoint;

        // 選択位置のXYZ座標を壁に保存し、 
        // 保存したデータを読み出します

        StoreDataInWall( wall, dataToStore );

        // 全てのスキーマをリストします

        ListSchemas();

        return Result.Succeeded;
      }
      catch( Exception ex )
      {
        message = ex.Message;
        return Result.Failed;
      }
    }
  }
}
